@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015 Jean-Pierre Charras, jaen-pierre.charras@gipsa-lab.inpg.com
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,33 +32,36 @@
 #include <wx/log.h>
 
 #include <wx/fdrepdlg.h>
+#include <eda_pattern_match.h>
 
-EDA_ITEM::EDA_ITEM( EDA_ITEM* parent, KICAD_T idType ) :
-        m_status( 0 ),
-        m_parent( parent ),
-        m_forceVisible( false ),
+EDA_ITEM::EDA_ITEM( EDA_ITEM* parent, KICAD_T idType, bool isSCH_ITEM, bool isBOARD_ITEM ) :
+        KIGFX::VIEW_ITEM( isSCH_ITEM, isBOARD_ITEM ),
+        m_structType( idType ),
         m_flags( 0 ),
-        m_structType( idType )
+        m_parent( parent ),
+        m_forceVisible( false )
 { }
 
 
-EDA_ITEM::EDA_ITEM( KICAD_T idType ) :
-        m_status( 0 ),
-        m_parent( nullptr ),
-        m_forceVisible( false ),
+EDA_ITEM::EDA_ITEM( KICAD_T idType, bool isSCH_ITEM, bool isBOARD_ITEM ) :
+        KIGFX::VIEW_ITEM( isSCH_ITEM, isBOARD_ITEM ),
+        m_structType( idType ),
         m_flags( 0 ),
-        m_structType( idType )
+        m_parent( nullptr ),
+        m_forceVisible( false )
 { }
 
 
 EDA_ITEM::EDA_ITEM( const EDA_ITEM& base ) :
+        KIGFX::VIEW_ITEM( base.IsSCH_ITEM(), base.IsBOARD_ITEM() ),
         m_Uuid( base.m_Uuid ),
-        m_status( base.m_status ),
-        m_parent( base.m_parent ),
-        m_forceVisible( base.m_forceVisible ),
+        m_structType( base.m_structType ),
         m_flags( base.m_flags ),
-        m_structType( base.m_structType )
-{ }
+        m_parent( base.m_parent ),
+        m_forceVisible( base.m_forceVisible )
+{
+    SetForcedTransparency( base.GetForcedTransparency() );
+}
 
 
 void EDA_ITEM::SetModified()
@@ -105,7 +108,7 @@ INSPECT_RESULT EDA_ITEM::Visit( INSPECTOR inspector, void* testData,
 }
 
 
-wxString EDA_ITEM::GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const
+wxString EDA_ITEM::GetItemDescription( UNITS_PROVIDER* aUnitsProvider, bool aFull ) const
 {
     wxFAIL_MSG( wxT( "GetItemDescription() was not overridden for schematic item type " ) +
                 GetClass() );
@@ -114,6 +117,12 @@ wxString EDA_ITEM::GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const
 }
 
 
+bool isWordChar( const wxUniChar& c )
+{
+    return wxIsalnum( c ) || c == '_';
+}
+
+
 bool EDA_ITEM::Matches( const wxString& aText, const EDA_SEARCH_DATA& aSearchData ) const
 {
     wxString text = aText;
@@ -129,7 +138,19 @@ bool EDA_ITEM::Matches( const wxString& aText, const EDA_SEARCH_DATA& aSearchDat
         searchText.MakeUpper();
     }
 
-    if( aSearchData.matchMode == EDA_SEARCH_MATCH_MODE::WHOLEWORD )
+    auto isWordChar =
+            []( const wxUniChar& c )
+            {
+                return wxIsalnum( c ) || c == '_';
+            };
+
+    if( aSearchData.matchMode == EDA_SEARCH_MATCH_MODE::PERMISSIVE )
+    {
+        EDA_COMBINED_MATCHER matcher( searchText, CTX_SEARCH );
+
+        return matcher.Find( text );
+    }
+    else if( aSearchData.matchMode == EDA_SEARCH_MATCH_MODE::WHOLEWORD )
     {
         int ii = 0;
 
@@ -143,8 +164,8 @@ bool EDA_ITEM::Matches( const wxString& aText, const EDA_SEARCH_DATA& aSearchDat
             ii = next;
             next += searchText.length();
 
-            bool startOK = ( ii == 0 || !wxIsalnum( text.GetChar( ii - 1 ) ) );
-            bool endOK = ( next == (int) text.length() || !wxIsalnum( text.GetChar( next ) ) );
+            bool startOK = ( ii == 0 || !isWordChar( text.GetChar( ii - 1 ) ) );
+            bool endOK = ( next == (int) text.length() || !isWordChar( text.GetChar( next ) ) );
 
             if( startOK && endOK )
                 return true;
@@ -201,8 +222,8 @@ bool EDA_ITEM::Replace( const EDA_SEARCH_DATA& aSearchData, wxString& aText )
 
         if( aSearchData.matchMode == EDA_SEARCH_MATCH_MODE::WHOLEWORD )
         {
-            startOK = ( ii == 0 || !wxIsalnum( text.GetChar( ii - 1 ) ) );
-            endOK = ( next == (int) text.length() || !wxIsalnum( text.GetChar( next ) ) );
+            startOK = ( ii == 0 || !isWordChar( text.GetChar( ii - 1 ) ) );
+            endOK = ( next == (int) text.length() || !isWordChar( text.GetChar( next ) ) );
         }
         else
         {
@@ -241,12 +262,13 @@ EDA_ITEM& EDA_ITEM::operator=( const EDA_ITEM& aItem )
 {
     // do not call initVars()
 
-    m_structType = aItem.m_structType;
-    m_flags      = aItem.m_flags;
-    m_status     = aItem.m_status;
-    m_parent     = aItem.m_parent;
+    m_structType   = aItem.m_structType;
+    m_flags        = aItem.m_flags;
+    m_parent       = aItem.m_parent;
     m_forceVisible = aItem.m_forceVisible;
 
+    SetForcedTransparency( aItem.GetForcedTransparency() );
+
     return *this;
 }
 
@@ -258,11 +280,11 @@ const BOX2I EDA_ITEM::ViewBBox() const
 }
 
 
-void EDA_ITEM::ViewGetLayers( int aLayers[], int& aCount ) const
+std::vector<int> EDA_ITEM::ViewGetLayers() const
 {
     // Basic fallback
-    aCount      = 1;
-    aLayers[0]  = 0;
+    std::vector<int> layers{ 1 };
+    return layers;
 }
 
 
@@ -325,18 +347,13 @@ static struct EDA_ITEM_DESC
             .Map( PCB_FOOTPRINT_T,         _HKI( "Footprint" ) )
             .Map( PCB_PAD_T,               _HKI( "Pad" ) )
             .Map( PCB_SHAPE_T,             _HKI( "Graphic" ) )
-            .Map( PCB_BITMAP_T,            _HKI( "Bitmap" ) )
+            .Map( PCB_REFERENCE_IMAGE_T,   _HKI( "Reference Image" ) )
+            .Map( PCB_GENERATOR_T,         _HKI( "Generator" ) )
+            .Map( PCB_FIELD_T,             _HKI( "Text" ) )
             .Map( PCB_TEXT_T,              _HKI( "Text" ) )
             .Map( PCB_TEXTBOX_T,           _HKI( "Text Box" ) )
-            .Map( PCB_FP_TEXT_T,           _HKI( "Text" ) )
-            .Map( PCB_FP_TEXTBOX_T,        _HKI( "Text Box" ) )
-            .Map( PCB_FP_SHAPE_T,          _HKI( "Graphic" ) )
-            .Map( PCB_FP_DIM_ALIGNED_T,    _HKI( "Dimension" ) )
-            .Map( PCB_FP_DIM_ORTHOGONAL_T, _HKI( "Dimension" ) )
-            .Map( PCB_FP_DIM_CENTER_T,     _HKI( "Dimension" ) )
-            .Map( PCB_FP_DIM_RADIAL_T,     _HKI( "Dimension" ) )
-            .Map( PCB_FP_DIM_LEADER_T,     _HKI( "Leader" ) )
-            .Map( PCB_FP_ZONE_T,           _HKI( "Zone" ) )
+            .Map( PCB_TABLE_T,             _HKI( "Table" ) )
+            .Map( PCB_TABLECELL_T,         _HKI( "Table Cell" ) )
             .Map( PCB_TRACE_T,             _HKI( "Track" ) )
             .Map( PCB_ARC_T,               _HKI( "Track" ) )
             .Map( PCB_VIA_T,               _HKI( "Via" ) )
@@ -359,8 +376,12 @@ static struct EDA_ITEM_DESC
             .Map( SCH_BUS_BUS_ENTRY_T,     _HKI( "Bus Entry" ) )
             .Map( SCH_LINE_T,              _HKI( "Line" ) )
             .Map( SCH_BITMAP_T,            _HKI( "Bitmap" ) )
+            .Map( SCH_SHAPE_T,             _HKI( "Graphic" ) )
+            .Map( SCH_RULE_AREA_T,         _HKI( "Rule Area" ) )
             .Map( SCH_TEXT_T,              _HKI( "Text" ) )
             .Map( SCH_TEXTBOX_T,           _HKI( "Text Box" ) )
+            .Map( SCH_TABLE_T,             _HKI( "Table" ) )
+            .Map( SCH_TABLECELL_T,         _HKI( "Table Cell" ) )
             .Map( SCH_LABEL_T,             _HKI( "Net Label" ) )
             .Map( SCH_DIRECTIVE_LABEL_T,   _HKI( "Directive Label" ) )
             .Map( SCH_GLOBAL_LABEL_T,      _HKI( "Global Label" ) )
@@ -379,11 +400,6 @@ static struct EDA_ITEM_DESC
             .Map( SCH_SCREEN_T,            _HKI( "SCH Screen" ) )
 
             .Map( LIB_SYMBOL_T,            _HKI( "Symbol" ) )
-            .Map( LIB_SHAPE_T,             _HKI( "Graphic" ) )
-            .Map( LIB_TEXT_T,              _HKI( "Text" ) )
-            .Map( LIB_TEXTBOX_T,           _HKI( "Text Box" ) )
-            .Map( LIB_PIN_T,               _HKI( "Pin" ) )
-            .Map( LIB_FIELD_T,             _HKI( "Symbol Field" ) )
 
             .Map( GERBER_LAYOUT_T,         _HKI( "Gerber Layout" ) )
             .Map( GERBER_DRAW_ITEM_T,      _HKI( "Draw Item" ) )
@@ -394,7 +410,7 @@ static struct EDA_ITEM_DESC
 
         propMgr.AddProperty( new PROPERTY_ENUM<EDA_ITEM, KICAD_T>( wxS( "Type" ),
                              NO_SETTER( EDA_ITEM, KICAD_T ), &EDA_ITEM::Type ) )
-                .SetIsInternal();
+                .SetIsHiddenFromPropertiesManager();
     }
 } _EDA_ITEM_DESC;
 
