@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2020 Jon Evans <jon@craftyjon.com>
- * Copyright (C) 2020-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -25,7 +25,9 @@
 #include <pgm_base.h>
 #include <settings/app_settings.h>
 #include <settings/json_settings_internals.h>
+#include <settings/color_settings.h>
 #include <settings/common_settings.h>
+#include <settings/grid_settings.h>
 #include <settings/parameters.h>
 
 
@@ -37,7 +39,9 @@ APP_SETTINGS_BASE::APP_SETTINGS_BASE( const std::string& aFilename, int aSchemaV
         m_ColorPicker(),
         m_LibTree(),
         m_Printing(),
+        m_SearchPane(),
         m_System(),
+        m_Plugins(),
         m_Window(),
         m_appSettingsSchemaVersion( aSchemaVersion )
 {
@@ -107,6 +111,9 @@ APP_SETTINGS_BASE::APP_SETTINGS_BASE( const std::string& aFilename, int aSchemaV
             },
             {} ) );
 
+    m_params.emplace_back(
+            new PARAM_LIST<wxString>( "lib_tree.open_libs", &m_LibTree.open_libs, {} ) );
+
     m_params.emplace_back( new PARAM<bool>( "printing.background",
             &m_Printing.background, false ) );
 
@@ -128,6 +135,10 @@ APP_SETTINGS_BASE::APP_SETTINGS_BASE( const std::string& aFilename, int aSchemaV
     m_params.emplace_back( new PARAM_LIST<int>( "printing.layers",
             &m_Printing.layers, {} ) );
 
+    m_params.emplace_back( new PARAM<int>( "search_pane.selection_zoom",
+            reinterpret_cast<int*>( &m_SearchPane.selection_zoom ),
+            static_cast<int>( SEARCH_PANE::SELECTION_ZOOM::PAN ) ) );
+
     m_params.emplace_back( new PARAM<bool>( "system.first_run_shown",
             &m_System.first_run_shown, false ) ); //@todo RFB remove? - not used
 
@@ -137,17 +148,62 @@ APP_SETTINGS_BASE::APP_SETTINGS_BASE( const std::string& aFilename, int aSchemaV
     m_params.emplace_back( new PARAM_LIST<wxString>( "system.file_history",
             &m_System.file_history, {} ) );
 
-    m_params.emplace_back( new PARAM<int>( "system.units",
-            &m_System.units, static_cast<int>( EDA_UNITS::MILLIMETRES ) ) );
+    if( m_filename == wxS( "pl_editor" )
+        || ( m_filename == wxS( "eeschema" ) || m_filename == wxS( "symbol_editor" ) ) )
+    {
+        m_params.emplace_back( new PARAM<int>( "system.units",
+                &m_System.units, static_cast<int>( EDA_UNITS::MILS ) ) );
+    }
+    else
+    {
+        m_params.emplace_back( new PARAM<int>( "system.units",
+                &m_System.units, static_cast<int>( EDA_UNITS::MM ) ) );
+    }
 
     m_params.emplace_back( new PARAM<int>( "system.last_metric_units",
-            &m_System.last_metric_units, static_cast<int>( EDA_UNITS::MILLIMETRES ) ) );
+            &m_System.last_metric_units, static_cast<int>( EDA_UNITS::MM ) ) );
 
     m_params.emplace_back( new PARAM<int>( "system.last_imperial_units",
-            &m_System.last_imperial_units, static_cast<int>( EDA_UNITS::INCHES ) ) );
+            &m_System.last_imperial_units, static_cast<int>( EDA_UNITS::MILS ) ) );
+
+    m_params.emplace_back( new PARAM<bool>( "system.show_import_issues",
+                                            &m_System.show_import_issues, true ) );
+
+    m_params.emplace_back( new PARAM_LAMBDA<nlohmann::json>( "plugins.actions",
+            [&]() -> nlohmann::json
+            {
+                nlohmann::json js = nlohmann::json::array();
+
+                for( const auto& [identifier, visible] : m_Plugins.actions )
+                    js.push_back( nlohmann::json( { { identifier.ToUTF8(), visible } } ) );
+
+                return js;
+            },
+            [&]( const nlohmann::json& aObj )
+            {
+                m_Plugins.actions.clear();
+
+                if( !aObj.is_array() )
+                {
+                    return;
+                }
+
+                for( const auto& entry : aObj )
+                {
+                    if( entry.empty() || !entry.is_object() )
+                        continue;
+
+                    for( const auto& pair : entry.items() )
+                    {
+                        m_Plugins.actions.emplace_back( std::make_pair(
+                                wxString( pair.key().c_str(), wxConvUTF8 ), pair.value() ) );
+                    }
+                }
+            },
+            nlohmann::json::array() ) );
 
     m_params.emplace_back( new PARAM<wxString>( "appearance.color_theme",
-            &m_ColorTheme, wxS( "_builtin_default" ) ) );
+            &m_ColorTheme, COLOR_SETTINGS::COLOR_BUILTIN_DEFAULT ) );
 
     addParamsForWindow( &m_Window, "window" );
 
@@ -269,8 +325,10 @@ bool APP_SETTINGS_BASE::migrateWindowConfig( wxConfigBase* aCfg, const std::stri
     ret &= fromLegacy<int>(  aCfg, aFrame + "Pos_x",                aJsonPath + ".pos_x" );
     ret &= fromLegacy<int>(  aCfg, aFrame + "Pos_y",                aJsonPath + ".pos_y" );
 
-    ret &= fromLegacy<bool>( aCfg, frameGDO + "ForceDisplayCursor", cursorPath + ".always_show_cursor" );
-    ret &= fromLegacy<bool>( aCfg, frameGDO + "CursorFullscreen",   cursorPath + ".fullscreen_cursor" );
+    ret &= fromLegacy<bool>( aCfg, frameGDO + "ForceDisplayCursor",
+                             cursorPath + ".always_show_cursor" );
+    ret &= fromLegacy<bool>( aCfg, frameGDO + "CursorFullscreen",
+                             cursorPath + ".fullscreen_cursor" );
 
     ret &= fromLegacy<int>(  aCfg, aFrame + "_LastGridSize",        gridPath + ".last_size" );
 
@@ -307,7 +365,8 @@ void APP_SETTINGS_BASE::addParamsForWindow( WINDOW_SETTINGS* aWindow, const std:
 
     m_params.emplace_back( new PARAM<int>( aJsonPath + ".pos_y", &aWindow->state.pos_y, 0 ) );
 
-    m_params.emplace_back( new PARAM<unsigned int>( aJsonPath + ".display", &aWindow->state.display, 0 ) );
+    m_params.emplace_back( new PARAM<unsigned int>( aJsonPath + ".display",
+                                                    &aWindow->state.display, 0 ) );
 
     m_params.emplace_back( new PARAM_LIST<double>( aJsonPath + ".zoom_factors",
             &aWindow->zoom_factors, {} ) );
@@ -317,27 +376,22 @@ void APP_SETTINGS_BASE::addParamsForWindow( WINDOW_SETTINGS* aWindow, const std:
 
     int defaultGridIdx;
 
-    if( m_filename == wxS( "pl_editor" ) )
+    if( ( m_filename == wxS( "eeschema" ) || m_filename == wxS( "symbol_editor" ) ) )
     {
         defaultGridIdx = 1;
-
-        m_params.emplace_back( new PARAM_LIST<wxString>( aJsonPath + ".grid.sizes",
-                &aWindow->grid.sizes, DefaultGridSizeList() ) );
     }
-    else if( m_filename == wxS( "eeschema" ) || m_filename == wxS( "symbol_editor" ) )
+    else if( m_filename == wxS( "pl_editor" ) )
     {
-        defaultGridIdx = 1;
-
-        // Eeschema's grids are fixed to keep wires/pins connected
+        defaultGridIdx = 4;
     }
     else
     {
-        defaultGridIdx = 4;
-
-        m_params.emplace_back( new PARAM_LIST<wxString>( aJsonPath + ".grid.sizes",
-                &aWindow->grid.sizes, DefaultGridSizeList() ) );
+        defaultGridIdx = 15;
     }
 
+    m_params.emplace_back( new PARAM_LIST<GRID>( aJsonPath + ".grid.sizes", &aWindow->grid.grids,
+                                                 DefaultGridSizeList() ) );
+
     m_params.emplace_back( new PARAM<int>( aJsonPath + ".grid.last_size",
             &aWindow->grid.last_size_idx, defaultGridIdx ) );
 
@@ -347,11 +401,65 @@ void APP_SETTINGS_BASE::addParamsForWindow( WINDOW_SETTINGS* aWindow, const std:
     m_params.emplace_back( new PARAM<int>( aJsonPath + ".grid.fast_grid_2",
             &aWindow->grid.fast_grid_2, defaultGridIdx + 1 ) );
 
-    // for grid user, use a default value compatible with eeschema and pcbnew (10 mils)
+    // legacy values, leave blank by default so we don't convert them
     m_params.emplace_back( new PARAM<wxString>( aJsonPath + ".grid.user_grid_x",
-            &aWindow->grid.user_grid_x, "10 mil" ) );
+            &aWindow->grid.user_grid_x, wxEmptyString ) );
     m_params.emplace_back( new PARAM<wxString>( aJsonPath + ".grid.user_grid_y",
-            &aWindow->grid.user_grid_y, "10 mil" ) );
+            &aWindow->grid.user_grid_y, wxEmptyString ) );
+
+    // for grid overrides, give just the schematic and symbol editors sane values
+    if( m_filename == wxS( "eeschema" ) || m_filename == wxS( "symbol_editor" ) )
+    {
+        m_params.emplace_back( new PARAM<bool>( aJsonPath + ".grid.overrides_enabled",
+                                                &aWindow->grid.overrides_enabled, true ) );
+        m_params.emplace_back( new PARAM<bool>( aJsonPath + ".grid.override_connected",
+                                                &aWindow->grid.override_connected, true ) );
+        m_params.emplace_back( new PARAM<bool>( aJsonPath + ".grid.override_wires",
+                                                &aWindow->grid.override_wires, true ) );
+        m_params.emplace_back( new PARAM<bool>( aJsonPath + ".grid.override_vias",
+                                                &aWindow->grid.override_vias, false ) );
+        m_params.emplace_back( new PARAM<bool>( aJsonPath + ".grid.override_text",
+                                                &aWindow->grid.override_text, true ) );
+        m_params.emplace_back( new PARAM<bool>( aJsonPath + ".grid.override_graphics",
+                                                &aWindow->grid.override_graphics, false ) );
+
+        m_params.emplace_back( new PARAM<int>( aJsonPath + ".grid.override_connected_idx",
+                                               &aWindow->grid.override_connected_idx, 1 ) );
+        m_params.emplace_back( new PARAM<int>( aJsonPath + ".grid.override_wires_idx",
+                                               &aWindow->grid.override_wires_idx, 1 ) );
+        m_params.emplace_back( new PARAM<int>( aJsonPath + ".grid.override_vias_idx",
+                                               &aWindow->grid.override_vias_idx, 0 ) );
+        m_params.emplace_back( new PARAM<int>( aJsonPath + ".grid.override_text_idx",
+                                               &aWindow->grid.override_text_idx, 3 ) );
+        m_params.emplace_back( new PARAM<int>( aJsonPath + ".grid.override_graphics_idx",
+                                               &aWindow->grid.override_graphics_idx, 2 ) );
+    }
+    else
+    {
+        m_params.emplace_back( new PARAM<bool>( aJsonPath + ".grid.overrides_enabled",
+                                                &aWindow->grid.overrides_enabled, true ) );
+        m_params.emplace_back( new PARAM<bool>( aJsonPath + ".grid.override_connected",
+                                                &aWindow->grid.override_connected, false ) );
+        m_params.emplace_back( new PARAM<bool>( aJsonPath + ".grid.override_wires",
+                                                &aWindow->grid.override_wires, false ) );
+        m_params.emplace_back( new PARAM<bool>( aJsonPath + ".grid.override_vias",
+                                                &aWindow->grid.override_vias, false ) );
+        m_params.emplace_back( new PARAM<bool>( aJsonPath + ".grid.override_text",
+                                                &aWindow->grid.override_text, false ) );
+        m_params.emplace_back( new PARAM<bool>( aJsonPath + ".grid.override_graphics",
+                                                &aWindow->grid.override_graphics, false ) );
+
+        m_params.emplace_back( new PARAM<int>( aJsonPath + ".grid.override_connected_idx",
+                                               &aWindow->grid.override_connected_idx, 16 ) );
+        m_params.emplace_back( new PARAM<int>( aJsonPath + ".grid.override_text_idx",
+                                               &aWindow->grid.override_text_idx, 18 ) );
+        m_params.emplace_back( new PARAM<int>( aJsonPath + ".grid.override_wires_idx",
+                                               &aWindow->grid.override_wires_idx, 19 ) );
+        m_params.emplace_back( new PARAM<int>( aJsonPath + ".grid.override_vias_idx",
+                                               &aWindow->grid.override_vias_idx, 18 ) );
+        m_params.emplace_back( new PARAM<int>( aJsonPath + ".grid.override_graphics_idx",
+                                               &aWindow->grid.override_graphics_idx, 15 ) );
+    }
 
     m_params.emplace_back( new PARAM<double>( aJsonPath + ".grid.line_width",
             &aWindow->grid.line_width, 1.0 ) );
@@ -376,42 +484,39 @@ void APP_SETTINGS_BASE::addParamsForWindow( WINDOW_SETTINGS* aWindow, const std:
 }
 
 
-const std::vector<wxString> APP_SETTINGS_BASE::DefaultGridSizeList() const
+const std::vector<GRID> APP_SETTINGS_BASE::DefaultGridSizeList() const
 {
     if( m_filename == wxS( "eeschema" ) || m_filename == wxS( "symbol_editor" ) )
     {
-        return { wxS( "100 mil" ),
-                 wxS( "50 mil" ),
-                 wxS( "25 mil" ),
-                 wxS( "10 mil" ),
-                 wxS( "5 mil" ),
-                 wxS( "2 mil" ),
-                 wxS( "1 mil" ) };
+        return { GRID{ wxEmptyString, wxS( "100 mil" ), wxS( "100 mil" ) },
+                 GRID{ wxEmptyString, wxS( "50 mil" ), wxS( "50 mil" ) },
+                 GRID{ wxEmptyString, wxS( "25 mil" ), wxS( "25 mil" ) },
+                 GRID{ wxEmptyString, wxS( "10 mil" ), wxS( "10 mil" ) } };
     }
     else
     {
-        return { wxS( "1000 mil" ),
-                 wxS( "500 mil" ),
-                 wxS( "250 mil" ),
-                 wxS( "200 mil" ),
-                 wxS( "100 mil" ),
-                 wxS( "50 mil" ),
-                 wxS( "25 mil" ),
-                 wxS( "20 mil" ),
-                 wxS( "10 mil" ),
-                 wxS( "5 mil" ),
-                 wxS( "2 mil" ),
-                 wxS( "1 mil" ),
-                 wxS( "5.0 mm" ),
-                 wxS( "2.5 mm" ),
-                 wxS( "1.0 mm" ),
-                 wxS( "0.5 mm" ),
-                 wxS( "0.25 mm" ),
-                 wxS( "0.2 mm" ),
-                 wxS( "0.1 mm" ),
-                 wxS( "0.05 mm" ),
-                 wxS( "0.025 mm" ),
-                 wxS( "0.01 mm" ) };
+        return { GRID{ wxEmptyString, wxS( "1000 mil" ), wxS( "1000 mil" ) },
+                 GRID{ wxEmptyString, wxS( "500 mil" ), wxS( "500 mil" ) },
+                 GRID{ wxEmptyString, wxS( "250 mil" ), wxS( "250 mil" ) },
+                 GRID{ wxEmptyString, wxS( "200 mil" ), wxS( "200 mil" ) },
+                 GRID{ wxEmptyString, wxS( "100 mil" ), wxS( "100 mil" ) },
+                 GRID{ wxEmptyString, wxS( "50 mil" ), wxS( "50 mil" ) },
+                 GRID{ wxEmptyString, wxS( "25 mil" ), wxS( "25 mil" ) },
+                 GRID{ wxEmptyString, wxS( "20 mil" ), wxS( "20 mil" ) },
+                 GRID{ wxEmptyString, wxS( "10 mil" ), wxS( "10 mil" ) },
+                 GRID{ wxEmptyString, wxS( "5 mil" ), wxS( "5 mil" ) },
+                 GRID{ wxEmptyString, wxS( "2 mil" ), wxS( "2 mil" ) },
+                 GRID{ wxEmptyString, wxS( "1 mil" ), wxS( "1 mil" ) },
+                 GRID{ wxEmptyString, wxS( "5.0 mm" ), wxS( "5.0 mm" ) },
+                 GRID{ wxEmptyString, wxS( "2.5 mm" ), wxS( "2.5 mm" ) },
+                 GRID{ wxEmptyString, wxS( "1.0 mm" ), wxS( "1.0 mm" ) },
+                 GRID{ wxEmptyString, wxS( "0.5 mm" ), wxS( "0.5 mm" ) },
+                 GRID{ wxEmptyString, wxS( "0.25 mm" ), wxS( "0.25 mm" ) },
+                 GRID{ wxEmptyString, wxS( "0.2 mm" ), wxS( "0.2 mm" ) },
+                 GRID{ wxEmptyString, wxS( "0.1 mm" ), wxS( "0.1 mm" ) },
+                 GRID{ wxEmptyString, wxS( "0.05 mm" ), wxS( "0.05 mm" ) },
+                 GRID{ wxEmptyString, wxS( "0.025 mm" ), wxS( "0.025 mm" ) },
+                 GRID{ wxEmptyString, wxS( "0.01 mm" ), wxS( "0.01 mm" ) } };
     }
 }
 
