@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2013 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2013 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,7 +29,6 @@
 #include <widgets/color_swatch.h>
 #include <settings/color_settings.h>
 #include <sch_edit_frame.h>
-#include <sch_validators.h>
 #include <tool/tool_manager.h>
 #include <gr_text.h>
 #include <confirm.h>
@@ -37,13 +36,13 @@
 #include <dialogs/html_message_box.h>
 #include <dialog_label_properties.h>
 #include <string_utils.h>
-#include <project/net_settings.h>
-#include <project/project_file.h>
 #include <kiface_base.h>
 #include <sch_label.h>
+#include <sch_commit.h>
 
 
-DIALOG_LABEL_PROPERTIES::DIALOG_LABEL_PROPERTIES( SCH_EDIT_FRAME* aParent, SCH_LABEL_BASE* aLabel ) :
+DIALOG_LABEL_PROPERTIES::DIALOG_LABEL_PROPERTIES( SCH_EDIT_FRAME* aParent,
+                                                  SCH_LABEL_BASE* aLabel ) :
         DIALOG_LABEL_PROPERTIES_BASE( aParent ),
         m_Parent( aParent ),
         m_currentLabel( aLabel ),
@@ -56,7 +55,7 @@ DIALOG_LABEL_PROPERTIES::DIALOG_LABEL_PROPERTIES( SCH_EDIT_FRAME* aParent, SCH_L
     COLOR_SETTINGS* colorSettings = m_Parent->GetColorSettings();
     COLOR4D         schematicBackground = colorSettings->GetColor( LAYER_SCHEMATIC_BACKGROUND );
 
-    m_fields = new FIELDS_GRID_TABLE<SCH_FIELD>( this, aParent, m_grid, m_currentLabel );
+    m_fields = new FIELDS_GRID_TABLE( this, aParent, m_grid, m_currentLabel );
     m_width = 100;  // Will be later set to a better value
     m_delayedFocusRow = -1;
     m_delayedFocusColumn = FDC_VALUE;
@@ -110,7 +109,7 @@ DIALOG_LABEL_PROPERTIES::DIALOG_LABEL_PROPERTIES( SCH_EDIT_FRAME* aParent, SCH_L
     m_grid->SetDefaultRowSize( m_grid->GetDefaultRowSize() + 4 );
 
     m_grid->SetTable( m_fields );
-    m_grid->PushEventHandler( new FIELDS_GRID_TRICKS( m_grid, this,
+    m_grid->PushEventHandler( new FIELDS_GRID_TRICKS( m_grid, this, nullptr,
                                                       [&]( wxCommandEvent& aEvent )
                                                       {
                                                           OnAddField( aEvent );
@@ -125,17 +124,17 @@ DIALOG_LABEL_PROPERTIES::DIALOG_LABEL_PROPERTIES( SCH_EDIT_FRAME* aParent, SCH_L
     }
 
     // Configure button logos
-    m_bpAdd->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
-    m_bpDelete->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
-    m_bpMoveUp->SetBitmap( KiBitmap( BITMAPS::small_up ) );
-    m_bpMoveDown->SetBitmap( KiBitmap( BITMAPS::small_down ) );
+    m_bpAdd->SetBitmap( KiBitmapBundle( BITMAPS::small_plus ) );
+    m_bpDelete->SetBitmap( KiBitmapBundle( BITMAPS::small_trash ) );
+    m_bpMoveUp->SetBitmap( KiBitmapBundle( BITMAPS::small_up ) );
+    m_bpMoveDown->SetBitmap( KiBitmapBundle( BITMAPS::small_down ) );
 
     m_separator1->SetIsSeparator();
 
     m_bold->SetIsCheckButton();
-    m_bold->SetBitmap( KiBitmap( BITMAPS::text_bold ) );
+    m_bold->SetBitmap( KiBitmapBundle( BITMAPS::text_bold ) );
     m_italic->SetIsCheckButton();
-    m_italic->SetBitmap( KiBitmap( BITMAPS::text_italic ) );
+    m_italic->SetBitmap( KiBitmapBundle( BITMAPS::text_italic ) );
 
     m_separator2->SetIsSeparator();
 
@@ -157,10 +156,10 @@ DIALOG_LABEL_PROPERTIES::DIALOG_LABEL_PROPERTIES( SCH_EDIT_FRAME* aParent, SCH_L
         m_diamond->Hide();
         m_rectangle->Hide();
 
-        m_spin0->SetBitmap( KiBitmap( BITMAPS::label_align_left ) );
-        m_spin1->SetBitmap( KiBitmap( BITMAPS::label_align_right ) );
-        m_spin2->SetBitmap( KiBitmap( BITMAPS::label_align_bottom ) );
-        m_spin3->SetBitmap( KiBitmap( BITMAPS::label_align_top ) );
+        m_spin0->SetBitmap( KiBitmapBundle( BITMAPS::label_align_left ) );
+        m_spin1->SetBitmap( KiBitmapBundle( BITMAPS::label_align_right ) );
+        m_spin2->SetBitmap( KiBitmapBundle( BITMAPS::label_align_bottom ) );
+        m_spin3->SetBitmap( KiBitmapBundle( BITMAPS::label_align_top ) );
     }
     else if( m_currentLabel->Type() == SCH_DIRECTIVE_LABEL_T )
     {
@@ -176,24 +175,25 @@ DIALOG_LABEL_PROPERTIES::DIALOG_LABEL_PROPERTIES( SCH_EDIT_FRAME* aParent, SCH_L
         m_bold->Hide();
         m_italic->Hide();
         m_separator2->Hide();
-        m_spin0->SetBitmap( KiBitmap( BITMAPS::pinorient_down ) );
-        m_spin1->SetBitmap( KiBitmap( BITMAPS::pinorient_up ) );
-        m_spin2->SetBitmap( KiBitmap( BITMAPS::pinorient_right ) );
-        m_spin3->SetBitmap( KiBitmap( BITMAPS::pinorient_left ) );
+        m_spin0->SetBitmap( KiBitmapBundle( BITMAPS::pinorient_down ) );
+        m_spin1->SetBitmap( KiBitmapBundle( BITMAPS::pinorient_up ) );
+        m_spin2->SetBitmap( KiBitmapBundle( BITMAPS::pinorient_right ) );
+        m_spin3->SetBitmap( KiBitmapBundle( BITMAPS::pinorient_left ) );
         m_separator3->Hide();
 
         m_formattingGB->Detach( m_fontCtrl );
         m_formattingGB->Detach( m_iconBar );
-        m_formattingGB->Add( m_iconBar, wxGBPosition( 0, 1 ), wxGBSpan( 1, 1 ), wxEXPAND|wxRIGHT, 5 );
+        m_formattingGB->Add( m_iconBar, wxGBPosition( 0, 1 ), wxGBSpan( 1, 1 ),
+                             wxEXPAND | wxRIGHT, 5 );
     }
     else
     {
         m_shapeSizer->Show( false );
 
-        m_spin0->SetBitmap( KiBitmap( BITMAPS::text_align_left ) );
-        m_spin1->SetBitmap( KiBitmap( BITMAPS::text_align_right ) );
-        m_spin2->SetBitmap( KiBitmap( BITMAPS::text_align_bottom ) );
-        m_spin3->SetBitmap( KiBitmap( BITMAPS::text_align_top ) );
+        m_spin0->SetBitmap( KiBitmapBundle( BITMAPS::text_align_left ) );
+        m_spin1->SetBitmap( KiBitmapBundle( BITMAPS::text_align_right ) );
+        m_spin2->SetBitmap( KiBitmapBundle( BITMAPS::text_align_bottom ) );
+        m_spin3->SetBitmap( KiBitmapBundle( BITMAPS::text_align_top ) );
     }
 
     if( !m_currentLabel->AutoRotateOnPlacementSupported() )
@@ -287,11 +287,12 @@ bool DIALOG_LABEL_PROPERTIES::TransferDataToWindow()
                     // Ensure the symbol has the Power (i.e. equivalent to a global label
                     // before adding its value in list
                     if( power->IsSymbolLikePowerGlobalLabel() )
-                        existingLabels.insert( UnescapeString( power->GetField( VALUE_FIELD )->GetText() ) );
+                        existingLabels.insert(
+                                UnescapeString( power->GetField( VALUE_FIELD )->GetText() ) );
                 }
             }
 
-            std::set<std::shared_ptr<BUS_ALIAS>> sheetAliases = screen->GetBusAliases();
+            auto& sheetAliases = screen->GetBusAliases();
             busAliases.insert( busAliases.end(), sheetAliases.begin(), sheetAliases.end() );
         }
 
@@ -319,7 +320,7 @@ bool DIALOG_LABEL_PROPERTIES::TransferDataToWindow()
     }
 
     // notify the grid
-    wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, m_fields->size() );
+    wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, (int) m_fields->size() );
     m_grid->ProcessTableMessage( msg );
     AdjustGridColumns( m_grid->GetRect().GetWidth() );
 
@@ -350,12 +351,12 @@ bool DIALOG_LABEL_PROPERTIES::TransferDataToWindow()
     m_italic->Check( m_currentLabel->IsItalic() );
     m_textColorSwatch->SetSwatchColor( m_currentLabel->GetTextColor(), false );
 
-    switch( m_currentLabel->GetTextSpinStyle() )
+    switch( m_currentLabel->GetSpinStyle() )
     {
-    case TEXT_SPIN_STYLE::RIGHT:  m_spin0->Check( true ); break;
-    case TEXT_SPIN_STYLE::LEFT:   m_spin1->Check( true ); break;
-    case TEXT_SPIN_STYLE::UP:     m_spin2->Check( true ); break;
-    case TEXT_SPIN_STYLE::BOTTOM: m_spin3->Check( true ); break;
+    case SPIN_STYLE::RIGHT:  m_spin0->Check( true ); break;
+    case SPIN_STYLE::LEFT:   m_spin1->Check( true ); break;
+    case SPIN_STYLE::UP:     m_spin2->Check( true ); break;
+    case SPIN_STYLE::BOTTOM: m_spin3->Check( true ); break;
     }
 
     if( m_currentLabel->AutoRotateOnPlacementSupported() )
@@ -365,9 +366,6 @@ bool DIALOG_LABEL_PROPERTIES::TransferDataToWindow()
 }
 
 
-/*!
- * wxEVT_COMMAND_ENTER event handler for single-line control
- */
 void DIALOG_LABEL_PROPERTIES::OnEnterKey( wxCommandEvent& aEvent )
 {
     wxPostEvent( this, wxCommandEvent( wxEVT_COMMAND_BUTTON_CLICKED, wxID_OK ) );
@@ -409,7 +407,7 @@ static bool positioningChanged( const SCH_FIELD& a, const SCH_FIELD& b )
 }
 
 
-static bool positioningChanged( FIELDS_GRID_TABLE<SCH_FIELD>* a, std::vector<SCH_FIELD>& b )
+static bool positioningChanged( FIELDS_GRID_TABLE* a, std::vector<SCH_FIELD>& b )
 {
     for( size_t i = 0; i < a->size() && i < b.size(); ++i )
     {
@@ -430,17 +428,15 @@ bool DIALOG_LABEL_PROPERTIES::TransferDataFromWindow()
         return false;
 
     // Don't allow text to disappear; it can be difficult to correct if you can't select it
-    if( !m_textSize.Validate( 0.01, 1000.0, EDA_UNITS::MILLIMETRES ) )
+    if( !m_textSize.Validate( 0.01, 1000.0, EDA_UNITS::MM ) )
         return false;
 
-    wxString text;
+    SCH_COMMIT commit( m_Parent );
+    wxString   text;
 
     /* save old text in undo list if not already in edit */
     if( m_currentLabel->GetEditFlags() == 0 )
-    {
-        m_Parent->SaveCopyInUndoList( m_Parent->GetScreen(), m_currentLabel, UNDO_REDO::CHANGED,
-                                      false );
-    }
+        commit.Modify( m_currentLabel, m_Parent->GetScreen() );
 
     m_Parent->GetCanvas()->Refresh();
 
@@ -466,8 +462,6 @@ bool DIALOG_LABEL_PROPERTIES::TransferDataFromWindow()
         m_currentLabel->SetText( text );
     }
 
-    bool doAutoplace = false;
-
     // change all field positions from relative to absolute
     for( SCH_FIELD& field : *m_fields )
     {
@@ -495,9 +489,7 @@ bool DIALOG_LABEL_PROPERTIES::TransferDataFromWindow()
     }
 
     if( positioningChanged( m_fields, m_currentLabel->GetFields() ) )
-        m_currentLabel->ClearFieldsAutoplaced();
-    else
-        doAutoplace = true;
+        m_currentLabel->SetFieldsAutoplaced( AUTOPLACE_NONE );
 
     for( int ii = m_fields->GetNumberRows() - 1; ii >= 0; ii-- )
     {
@@ -506,23 +498,41 @@ bool DIALOG_LABEL_PROPERTIES::TransferDataFromWindow()
         const wxString& fieldText = field.GetText();
 
         if( fieldName.IsEmpty() && fieldText.IsEmpty() )
+        {
+            // delete empty, unnamed fields
             m_fields->erase( m_fields->begin() + ii );
+        }
         else if( fieldName == wxT( "Netclass" ) && fieldText.IsEmpty() )
-            m_fields->erase( m_fields->begin() + ii );
+        {
+            // delete empty Netclass fields if there are other Netclass fields present
+            int netclassFieldCount = 0;
+
+            for( int jj = 0; jj < m_fields->GetNumberRows(); ++jj )
+            {
+                if( m_fields->at( jj ).GetCanonicalName() == wxT( "Netclass" ) )
+                    netclassFieldCount++;
+            }
+
+            if( netclassFieldCount > 1 )
+                m_fields->erase( m_fields->begin() + ii );
+        }
         else if( fieldName.IsEmpty() )
+        {
+            // give non-empty, unnamed fields a name
             field.SetName( _( "untitled" ) );
+        }
     }
 
     m_currentLabel->SetFields( *m_fields );
 
     if( m_shapeSizer->AreAnyItemsShown() )
     {
-        if( m_input->GetValue() )
+        if( m_bidirectional->GetValue() )
+            m_currentLabel->SetShape( LABEL_FLAG_SHAPE::L_BIDI );
+        else if( m_input->GetValue() )
             m_currentLabel->SetShape( LABEL_FLAG_SHAPE::L_INPUT );
         else if( m_output->GetValue() )
             m_currentLabel->SetShape( LABEL_FLAG_SHAPE::L_OUTPUT );
-        else if( m_bidirectional->GetValue() )
-            m_currentLabel->SetShape( LABEL_FLAG_SHAPE::L_BIDI );
         else if( m_triState->GetValue() )
             m_currentLabel->SetShape( LABEL_FLAG_SHAPE::L_TRISTATE );
         else if( m_passive->GetValue() )
@@ -544,38 +554,23 @@ bool DIALOG_LABEL_PROPERTIES::TransferDataFromWindow()
     }
 
     if( m_currentLabel->Type() == SCH_DIRECTIVE_LABEL_T )
-        static_cast<SCH_DIRECTIVE_LABEL*>( m_currentLabel )->SetPinLength( m_textSize.GetValue() );
-    else if( m_currentLabel->GetTextWidth() != m_textSize.GetValue() )
-        m_currentLabel->SetTextSize( wxSize( m_textSize.GetValue(), m_textSize.GetValue() ) );
-
-    if( m_bold->IsChecked() != m_currentLabel->IsBold() )
-    {
-        if( m_bold->IsChecked() )
-        {
-            m_currentLabel->SetBold( true );
-            m_currentLabel->SetTextThickness( GetPenSizeForBold( m_currentLabel->GetTextWidth() ) );
-        }
-        else
-        {
-            m_currentLabel->SetBold( false );
-            m_currentLabel->SetTextThickness( 0 ); // Use default pen width
-        }
-    }
+        static_cast<SCH_DIRECTIVE_LABEL*>( m_currentLabel )->SetPinLength( m_textSize.GetIntValue() );
+    else if( m_currentLabel->GetTextWidth() != m_textSize.GetIntValue() )
+        m_currentLabel->SetTextSize( VECTOR2I( m_textSize.GetIntValue(),
+                                               m_textSize.GetIntValue() ) );
 
+    // Must come after SetTextSize()
+    m_currentLabel->SetBold( m_bold->IsChecked() );
     m_currentLabel->SetItalic( m_italic->IsChecked() );
 
     m_currentLabel->SetTextColor( m_textColorSwatch->GetSwatchColor() );
 
-    TEXT_SPIN_STYLE selectedSpinStyle= TEXT_SPIN_STYLE::LEFT;
+    SPIN_STYLE selectedSpinStyle= SPIN_STYLE::LEFT;
 
-    if( m_spin0->IsChecked() )
-        selectedSpinStyle = TEXT_SPIN_STYLE::RIGHT;
-    else if( m_spin1->IsChecked() )
-        selectedSpinStyle = TEXT_SPIN_STYLE::LEFT;
-    else if( m_spin2->IsChecked() )
-        selectedSpinStyle = TEXT_SPIN_STYLE::UP;
-    else if( m_spin3->IsChecked() )
-        selectedSpinStyle = TEXT_SPIN_STYLE::BOTTOM;
+    if( m_spin0->IsChecked() )      selectedSpinStyle = SPIN_STYLE::RIGHT;
+    else if( m_spin1->IsChecked() ) selectedSpinStyle = SPIN_STYLE::LEFT;
+    else if( m_spin2->IsChecked() ) selectedSpinStyle = SPIN_STYLE::UP;
+    else if( m_spin3->IsChecked() ) selectedSpinStyle = SPIN_STYLE::BOTTOM;
 
     if( m_currentLabel->AutoRotateOnPlacementSupported() )
     {
@@ -584,17 +579,23 @@ bool DIALOG_LABEL_PROPERTIES::TransferDataFromWindow()
         frame->AutoRotateItem( frame->GetScreen(), m_currentLabel );
     }
     else
+    {
         m_currentLabel->SetAutoRotateOnPlacement( false );
+    }
 
     if( !m_currentLabel->AutoRotateOnPlacement()
-        && m_currentLabel->GetTextSpinStyle() != selectedSpinStyle )
-        m_currentLabel->SetTextSpinStyle( selectedSpinStyle );
+        && m_currentLabel->GetSpinStyle() != selectedSpinStyle )
+    {
+        m_currentLabel->SetSpinStyle( selectedSpinStyle );
+    }
 
-    if( doAutoplace )
-        m_currentLabel->AutoAutoplaceFields( m_Parent->GetScreen() );
+    AUTOPLACE_ALGO fieldsAutoplaced = m_currentLabel->GetFieldsAutoplaced();
 
-    m_Parent->UpdateItem( m_currentLabel );
-    m_Parent->GetCanvas()->Refresh();
+    if( fieldsAutoplaced == AUTOPLACE_AUTO || fieldsAutoplaced == AUTOPLACE_MANUAL )
+        m_currentLabel->AutoplaceFields( m_Parent->GetScreen(), fieldsAutoplaced );
+
+    if( !commit.Empty() )
+        commit.Push( _( "Edit Label Properties" ) );
 
     return true;
 }
@@ -634,7 +635,7 @@ void DIALOG_LABEL_PROPERTIES::OnAddField( wxCommandEvent& event )
         fieldName = SCH_LABEL_BASE::GetDefaultFieldName( fieldName, true );
     }
 
-    SCH_FIELD newField( wxPoint( 0, 0 ), fieldID, m_currentLabel, fieldName );
+    SCH_FIELD newField( VECTOR2I( 0, 0 ), fieldID, m_currentLabel, fieldName );
 
     if( m_fields->size() > 0 )
     {
@@ -685,10 +686,15 @@ void DIALOG_LABEL_PROPERTIES::OnDeleteField( wxCommandEvent& event )
     m_grid->CommitPendingChanges( true /* quiet mode */ );
 
     // Reverse sort so deleting a row doesn't change the indexes of the other rows.
-    selectedRows.Sort( []( int* first, int* second ) { return *second - *first; } );
+    selectedRows.Sort( []( int* first, int* second )
+                       {
+                           return *second - *first;
+                       } );
 
     for( int row : selectedRows )
     {
+        //avoids an assert if we deselect early here
+        m_grid->DeselectRow( row );
         m_fields->erase( m_fields->begin() + row );
 
         // notify the grid
