@@ -61,9 +61,9 @@ std::string NAME_GENERATOR::Generate( const std::string& aProposedName )
 }
 
 
-NETLIST_EXPORTER_SPICE::NETLIST_EXPORTER_SPICE( SCHEMATIC_IFACE* aSchematic ) :
+NETLIST_EXPORTER_SPICE::NETLIST_EXPORTER_SPICE( SCHEMATIC* aSchematic ) :
     NETLIST_EXPORTER_BASE( aSchematic ),
-    m_libMgr( &aSchematic->Prj() )
+    m_libMgr( &aSchematic->Prj(), aSchematic )
 {
 }
 
@@ -455,7 +455,7 @@ void NETLIST_EXPORTER_SPICE::readModel( SCH_SHEET_PATH& aSheet, SCH_SYMBOL& aSym
         auto spiceGenerator = static_cast<const SPICE_GENERATOR_IBIS&>( ibisModel->SpiceGenerator() );
 
         wxString    cacheFilepath = cacheFn.GetPath( wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR );
-        std::string modelData = spiceGenerator.IbisDevice( aItem, m_schematic->Prj(),
+        std::string modelData = spiceGenerator.IbisDevice( aItem, m_schematic,
                                                            cacheFilepath, aReporter );
 
         cacheFile.Write( wxString( modelData ) );
