@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,7 +29,7 @@
 
 class KIID_PATH;
 class SCH_SCREEN;
-class SCH_SEXPR_PARSER;
+class SCH_IO_KICAD_SEXPR_PARSER;
 class SCH_SHEET_LIST;
 class SCH_SHEET_PIN;
 class SCH_SHEET_PATH;
@@ -46,9 +46,11 @@ enum  SHEET_FIELD_TYPE
     SHEETFILENAME,
 
     /// The first 2 are mandatory, and must be instantiated in SCH_SHEET
-    SHEET_MANDATORY_FIELDS
+    SHEET_MANDATORY_FIELD_COUNT
 };
 
+#define SHEET_MANDATORY_FIELDS { SHEETNAME, SHEETFILENAME }
+
 
 /**
  * Sheet symbol placed in a schematic, and is the entry point for a sub schematic.
@@ -57,13 +59,14 @@ class SCH_SHEET : public SCH_ITEM
 {
 public:
     SCH_SHEET( EDA_ITEM* aParent = nullptr, const VECTOR2I& aPos = VECTOR2I( 0, 0 ),
-               wxSize aSize = wxSize( schIUScale.MilsToIU( MIN_SHEET_WIDTH ),
-                                      schIUScale.MilsToIU( MIN_SHEET_HEIGHT ) ),
-               FIELDS_AUTOPLACED aAutoplaceFields = FIELDS_AUTOPLACED_AUTO );
+               VECTOR2I aSize = VECTOR2I( schIUScale.MilsToIU( MIN_SHEET_WIDTH ),
+                                          schIUScale.MilsToIU( MIN_SHEET_HEIGHT ) ) );
 
     /**
-     * Copy \a aSheet into a new object.  All sheet pins are copied as is except and
-     * the SCH_SHEET_PIN's #m_Parent pointers are set to the new copied parent object.
+     * Copy \a aSheet into a new object.
+     *
+     * All sheet pins are copied as is except and the #SCH_SHEET_PIN object's #m_Parent pointers
+     * are set to the new copied parent object.
      */
     SCH_SHEET( const SCH_SHEET& aSheet );
 
@@ -100,13 +103,17 @@ public:
      */
     void SetFields( const std::vector<SCH_FIELD>& aFields );
 
+    wxString GetShownName( bool aAllowExtraText ) const
+    {
+        return m_fields[SHEETNAME].GetShownText( aAllowExtraText );
+    }
     wxString GetName() const { return m_fields[ SHEETNAME ].GetText(); }
     void SetName( const wxString& aName ) { m_fields[ SHEETNAME ].SetText( aName ); }
 
     SCH_SCREEN* GetScreen() const { return m_screen; }
 
-    wxSize GetSize() const { return m_size; }
-    void SetSize( const wxSize& aSize ) { m_size = aSize; }
+    VECTOR2I GetSize() const { return m_size; }
+    void     SetSize( const VECTOR2I& aSize ) { m_size = aSize; }
 
     int GetBorderWidth() const { return m_borderWidth; }
     void SetBorderWidth( int aWidth ) { m_borderWidth = aWidth; }
@@ -152,7 +159,7 @@ public:
      *
      * @param aDepth is a counter to limit recursion and circular references.
      */
-    bool ResolveTextVar( wxString* token, int aDepth = 0 ) const;
+    bool ResolveTextVar( const SCH_SHEET_PATH* aPath, wxString* token, int aDepth = 0 ) const;
 
     void GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>& aList ) override;
 
@@ -161,6 +168,8 @@ public:
      */
     bool IsVerticalOrientation() const;
 
+    void SetPositionIgnoringPins( const VECTOR2I& aPosition );
+
     /**
      * Add aSheetPin to the sheet.
      *
@@ -200,7 +209,7 @@ public:
     SCH_SHEET_PIN* GetPin( const VECTOR2I& aPosition );
 
     /**
-     * Checks if the sheet already has a sheet pin named \a aName.
+     * Check if the sheet already has a sheet pin named \a aName.
      *
      * @param aName Name of the sheet pin to search for.
      * @return  True if sheet pin with \a aName is found, otherwise false.
@@ -243,8 +252,6 @@ public:
 
     int GetPenWidth() const override;
 
-    void Print( const RENDER_SETTINGS* aSettings, const VECTOR2I& aOffset ) override;
-
     /**
      * Return a bounding box for the sheet body but not the fields.
      */
@@ -254,8 +261,9 @@ public:
 
     /**
      * Rotating around the boundingBox's center can cause walking when the sheetname or
-     * filename is longer than the edge it's on.  Use this instead, which always returns
-     * the center of the sheet itself.
+     * filename is longer than the edge it's on.
+     *
+     * Use this instead, which always returns the center of the sheet itself.
      */
     VECTOR2I GetRotationCenter() const;
 
@@ -318,7 +326,7 @@ public:
     void Move( const VECTOR2I& aMoveVector ) override;
     void MirrorHorizontally( int aCenter ) override;
     void MirrorVertically( int aCenter ) override;
-    void Rotate( const VECTOR2I& aCenter ) override;
+    void Rotate( const VECTOR2I& aCenter, bool aRotateCCW ) override;
 
     bool Matches( const EDA_SEARCH_DATA& aSearchData, void* aAuxData ) const override;
 
@@ -329,17 +337,21 @@ public:
      *
      * @param[in] aSize The new size for this sheet.
      */
-    void Resize( const wxSize& aSize );
+    void Resize( const VECTOR2I& aSize );
 
-    void AutoplaceFields( SCH_SCREEN* aScreen, bool aManual ) override;
+    void AutoplaceFields( SCH_SCREEN* aScreen, AUTOPLACE_ALGO aAlgo ) override;
 
     void GetEndPoints( std::vector <DANGLING_END_ITEM>& aItemList ) override;
 
-    bool UpdateDanglingState( std::vector<DANGLING_END_ITEM>& aItemList,
-                              const SCH_SHEET_PATH* aPath = nullptr ) override;
+    bool UpdateDanglingState( std::vector<DANGLING_END_ITEM>& aItemListByType,
+                              std::vector<DANGLING_END_ITEM>& aItemListByPos,
+                              const SCH_SHEET_PATH*           aPath = nullptr ) override;
 
     bool IsConnectable() const override { return true; }
 
+    bool HasConnectivityChanges( const SCH_ITEM* aItem,
+                                 const SCH_SHEET_PATH* aInstance = nullptr ) const override;
+
     bool CanConnect( const SCH_ITEM* aItem ) const override
     {
         return ( aItem->Type() == SCH_LINE_T && aItem->GetLayer() == LAYER_WIRE )
@@ -355,7 +367,31 @@ public:
 
     void RunOnChildren( const std::function<void( SCH_ITEM* )>& aFunction ) override;
 
-    wxString GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const override;
+    /**
+     * Set or clear the exclude from simulation flag.
+     */
+    void SetExcludedFromSim( bool aExcludeFromSim ) override { m_excludedFromSim = aExcludeFromSim; }
+    bool GetExcludedFromSim() const override { return m_excludedFromSim; }
+
+    /**
+     * Set or clear the exclude from schematic bill of materials flag.
+     */
+    void SetExcludedFromBOM( bool aExcludeFromBOM ) { m_excludedFromBOM = aExcludeFromBOM; }
+    bool GetExcludedFromBOM() const { return m_excludedFromBOM; }
+
+    /**
+     * Set or clear exclude from board netlist flag.
+     */
+    void SetExcludedFromBoard( bool aExcludeFromBoard ) { m_excludedFromBoard = aExcludeFromBoard; }
+    bool GetExcludedFromBoard() const { return m_excludedFromBoard; }
+
+    /**
+     * Set or clear the 'Do Not Populate' flags
+     */
+    bool GetDNP() const { return m_DNP; }
+    void SetDNP( bool aDNP ) { m_DNP = aDNP; }
+
+    wxString GetItemDescription( UNITS_PROVIDER* aUnitsProvider, bool aFull ) const override;
 
     BITMAPS GetMenuImage() const override;
 
@@ -363,7 +399,7 @@ public:
 
     bool operator <( const SCH_ITEM& aItem ) const override;
 
-    void ViewGetLayers( int aLayers[], int& aCount ) const override;
+    std::vector<int> ViewGetLayers() const override;
 
     VECTOR2I GetPosition() const override { return m_pos; }
     void     SetPosition( const VECTOR2I& aPosition ) override;
@@ -371,7 +407,14 @@ public:
     bool HitTest( const VECTOR2I& aPosition, int aAccuracy ) const override;
     bool HitTest( const BOX2I& aRect, bool aContained, int aAccuracy = 0 ) const override;
 
-    void Plot( PLOTTER* aPlotter, bool aBackground ) const override;
+    void Print( const SCH_RENDER_SETTINGS* aSettings, int aUnit, int aBodyStyle,
+                const VECTOR2I& aOffset, bool aForceNoFill, bool aDimmed ) override;
+
+    void PrintBackground( const SCH_RENDER_SETTINGS* aSettings, int aUnit, int aBodyStyle,
+                          const VECTOR2I& aOffset, bool aDimmed ) override {}
+
+    void Plot( PLOTTER* aPlotter, bool aBackground, const SCH_PLOT_OPTS& aPlotOpts,
+               int aUnit, int aBodyStyle, const VECTOR2I& aOffset, bool aDimmed ) override;
 
     EDA_ITEM* Clone() const override;
 
@@ -401,22 +444,38 @@ public:
      */
     const SCH_SHEET_INSTANCE& GetRootInstance() const;
 
+    void RemoveInstance( const KIID_PATH& aInstancePath );
+
+    void AddInstance( const SCH_SHEET_INSTANCE& aInstance );
+
     /**
-     * Compares page numbers of schematic sheets.
+     * Check if the instance data of this sheet has any changes compared to \a aOther.
+     *
+     * @param aOther is the sheet to compare against.
+     * @return true if there are page number changes with \a aOther otherwise false.
+     */
+    bool HasPageNumberChanges( const SCH_SHEET& aOther ) const;
+
+    /**
+     * Compare page numbers of schematic sheets.
      *
      * @return 0 if the page numbers are equal, -1 if aPageNumberA < aPageNumberB, 1 otherwise
      */
     static int ComparePageNum( const wxString& aPageNumberA, const wxString& aPageNumberB );
 
+    double Similarity( const SCH_ITEM& aOther ) const override;
+
+    bool operator==( const SCH_ITEM& aOther ) const override;
+
 #if defined(DEBUG)
     void Show( int nestLevel, std::ostream& os ) const override;
 #endif
 
-    static const wxString GetDefaultFieldName( int aFieldNdx, bool aTranslated = true );
+    static const wxString GetDefaultFieldName( int aFieldNdx, bool aTranslated );
 
 protected:
     friend SCH_SHEET_PATH;
-    friend SCH_SEXPR_PARSER;
+    friend SCH_IO_KICAD_SEXPR_PARSER;
 
     void setInstances( const std::vector<SCH_SHEET_INSTANCE>& aInstances )
     {
@@ -430,40 +489,34 @@ protected:
      * in the list, do nothing.  Sheet instances allow for the sharing in complex hierarchies
      * which allows for per instance data such as page number for sheets to stored.
      *
-     * @warning The #SCH_SHEET_PATH object must be a full hierarchical path which means the
-     *          #SCH_SHEET object at index 0 must be the root sheet.  A partial sheet path
-     *          will raise an assertion on debug builds and silently fail and return false
-     *          on release builds.
+     * @warning The #KIID_PATH object must be a full hierarchical path which means the sheet
+     *          at index 0 must be the root sheet.
      *
      * @param[in] aInstance is the #SCH_SHEET_PATH of the sheet instance to the instance list.
      * @return false if the instance already exists, true if the instance was added.
      */
-    bool addInstance( const SCH_SHEET_PATH& aInstance );
+    bool addInstance( const KIID_PATH& aInstance );
 
     /**
      * Return the sheet page number for \a aInstance.
      *
-     * @warning The #SCH_SHEET_PATH object must be a full hierarchical path which means the
-     *          #SCH_SHEET object at index 0 must be the root sheet.  A partial sheet path
-     *          will raise an assertion on debug builds and silently fail and return an empty
-     *          page number on release builds.
+     * @warning The #KIID_PATH object must be a full hierarchical path which means the sheet
+     *          at index 0 must be the root sheet.
      *
      * @return the page number for the requested sheet instance.
      */
-    wxString getPageNumber( const SCH_SHEET_PATH& aInstance ) const;
+    wxString getPageNumber( const KIID_PATH& aInstance ) const;
 
     /**
      * Set the page number for the sheet instance \a aInstance.
      *
-     * @warning The #SCH_SHEET_PATH object must be a full hierarchical path which means the
-     *          #SCH_SHEET object at index 0 must be the root sheet.  A partial sheet path
-     *          will raise an assertion on debug builds and silently fail and return on release
-     *          builds.
+     * @warning The #KIID_PATH object must be a full hierarchical path which means the sheet
+     *          at index 0 must be the root sheet.
      *
      * @param[in] aInstance is the hierarchical path of the sheet.
      * @param[in] aReference is the new page number for the sheet.
      */
-    void setPageNumber( const SCH_SHEET_PATH& aInstance, const wxString& aPageNumber );
+    void setPageNumber( const KIID_PATH& aInstance, const wxString& aPageNumber );
 
     bool getInstance( SCH_SHEET_INSTANCE& aInstance, const KIID_PATH& aSheetPath,
                       bool aTestFromEnd = false ) const;
@@ -497,8 +550,13 @@ private:
     std::vector<SCH_SHEET_PIN*> m_pins;         // The list of sheet connection points.
     std::vector<SCH_FIELD>      m_fields;
 
+    bool                        m_excludedFromSim;
+    bool                        m_excludedFromBOM;
+    bool                        m_excludedFromBoard;
+    bool                        m_DNP;
+
     VECTOR2I                    m_pos;          // The position of the sheet.
-    wxSize                      m_size;         // The size of the sheet.
+    VECTOR2I                    m_size;         // The size of the sheet.
     int                         m_borderWidth;
     KIGFX::COLOR4D              m_borderColor;
     KIGFX::COLOR4D              m_backgroundColor;
