@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2019 CERN
- * Copyright (C) 2019-2024 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,9 +28,9 @@
 #include <gal/graphics_abstraction_layer.h>
 #include <tool/tool_manager.h>
 #include <tools/ee_grid_helper.h>
-#include <tools/ee_selection_tool.h>
+#include <tools/sch_selection_tool.h>
 #include <tools/sch_line_wire_bus_tool.h>
-#include <ee_actions.h>
+#include <sch_actions.h>
 #include <sch_commit.h>
 #include <eda_item.h>
 #include <sch_item.h>
@@ -42,6 +42,7 @@
 #include <sch_edit_frame.h>
 #include <eeschema_id.h>
 #include <pgm_base.h>
+#include <view/view_controls.h>
 #include <settings/settings_manager.h>
 #include "sch_move_tool.h"
 
@@ -51,7 +52,7 @@
 
 
 SCH_MOVE_TOOL::SCH_MOVE_TOOL() :
-        EE_TOOL_BASE<SCH_EDIT_FRAME>( "eeschema.InteractiveMove" ),
+        SCH_TOOL_BASE<SCH_EDIT_FRAME>( "eeschema.InteractiveMove" ),
         m_inMoveTool( false ),
         m_moveInProgress( false ),
         m_isDrag( false ),
@@ -62,7 +63,7 @@ SCH_MOVE_TOOL::SCH_MOVE_TOOL() :
 
 bool SCH_MOVE_TOOL::Init()
 {
-    EE_TOOL_BASE::Init();
+    SCH_TOOL_BASE::Init();
 
     auto moveCondition =
             []( const SELECTION& aSel )
@@ -80,9 +81,9 @@ bool SCH_MOVE_TOOL::Init()
     //
     CONDITIONAL_MENU& selToolMenu = m_selectionTool->GetToolMenu().GetMenu();
 
-    selToolMenu.AddItem( EE_ACTIONS::move, moveCondition, 150 );
-    selToolMenu.AddItem( EE_ACTIONS::drag, moveCondition, 150 );
-    selToolMenu.AddItem( EE_ACTIONS::alignToGrid, moveCondition, 150 );
+    selToolMenu.AddItem( SCH_ACTIONS::move, moveCondition, 150 );
+    selToolMenu.AddItem( SCH_ACTIONS::drag, moveCondition, 150 );
+    selToolMenu.AddItem( SCH_ACTIONS::alignToGrid, moveCondition, 150 );
 
     return true;
 }
@@ -370,7 +371,7 @@ void SCH_MOVE_TOOL::orthoLineDrag( SCH_COMMIT* aCommit, SCH_LINE* line, const VE
 
 int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
 {
-    m_isDrag = aEvent.IsAction( &EE_ACTIONS::drag );
+    m_isDrag = aEvent.IsAction( &SCH_ACTIONS::drag );
 
     if( SCH_COMMIT* commit = dynamic_cast<SCH_COMMIT*>( aEvent.Commit() ) )
     {
@@ -403,17 +404,8 @@ int SCH_MOVE_TOOL::Main( const TOOL_EVENT& aEvent )
 
 bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aCommit, bool aIsSlice )
 {
-    EESCHEMA_SETTINGS*    cfg = nullptr;
-
-    try
-    {
-        cfg = Pgm().GetSettingsManager().GetAppSettings<EESCHEMA_SETTINGS>();
-    }
-    catch( const std::runtime_error& e )
-    {
-        wxCHECK_MSG( false, false, e.what() );
-    }
-
+    SETTINGS_MANAGER&     mgr = Pgm().GetSettingsManager();
+    EESCHEMA_SETTINGS*    cfg = mgr.GetAppSettings<EESCHEMA_SETTINGS>( "eeschema" );
     KIGFX::VIEW_CONTROLS* controls = getViewControls();
     EE_GRID_HELPER        grid( m_toolMgr );
     bool                  wasDragging = m_moveInProgress && m_isDrag;
@@ -439,7 +431,7 @@ bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aComm
 
                 // And give it a kick so it doesn't have to wait for the first mouse movement
                 // to refresh.
-                m_toolMgr->PostAction( EE_ACTIONS::restartMove );
+                m_toolMgr->PostAction( SCH_ACTIONS::restartMove );
             }
         }
         else
@@ -456,10 +448,30 @@ bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aComm
 
     REENTRANCY_GUARD guard( &m_inMoveTool );
 
+    SCH_SELECTION& userSelection = m_selectionTool->GetSelection();
+
+    // If a single pin is selected, promote the move selection to its parent symbol
+    if( userSelection.GetSize() == 1 )
+    {
+        EDA_ITEM* selItem = userSelection.Front();
+
+        if( selItem->Type() == SCH_PIN_T )
+        {
+            EDA_ITEM* parent = selItem->GetParent();
+
+            if( parent->Type() == SCH_SYMBOL_T )
+            {
+                m_selectionTool->ClearSelection();
+                m_selectionTool->AddItemToSel( parent );
+            }
+        }
+    }
+
     // Be sure that there is at least one item that we can move. If there's no selection try
     // looking for the stuff under mouse cursor (i.e. Kicad old-style hover selection).
-    EE_SELECTION& selection = m_selectionTool->RequestSelection( EE_COLLECTOR::MovableItems );
-    bool          unselect = selection.IsHover();
+    SCH_SELECTION& selection = m_selectionTool->RequestSelection( SCH_COLLECTOR::MovableItems,
+                                                                  true );
+    bool           unselect = selection.IsHover();
 
     // Keep an original copy of the starting points for cleanup after the move
     std::vector<DANGLING_END_ITEM> internalPoints;
@@ -494,9 +506,9 @@ bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aComm
         grid.SetSnap( !evt->Modifier( MD_SHIFT ) );
         grid.SetUseGrid( getView()->GetGAL()->GetGridSnapping() && !evt->DisableGridSnapping() );
 
-        if( evt->IsAction( &EE_ACTIONS::restartMove )
-                || evt->IsAction( &EE_ACTIONS::move )
-                || evt->IsAction( &EE_ACTIONS::drag )
+        if( evt->IsAction( &SCH_ACTIONS::restartMove )
+                || evt->IsAction( &SCH_ACTIONS::move )
+                || evt->IsAction( &SCH_ACTIONS::drag )
                 || evt->IsMotion()
                 || evt->IsDrag( BUT_LEFT )
                 || evt->IsAction( &ACTIONS::refreshPreview ) )
@@ -637,7 +649,7 @@ bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aComm
                 //
                 m_cursor = controls->GetCursorPosition();
 
-                if( evt->IsAction( &EE_ACTIONS::restartMove ) )
+                if( evt->IsAction( &SCH_ACTIONS::restartMove ) )
                 {
                     wxASSERT_MSG( m_anchorPos, "Should be already set from previous cmd" );
                 }
@@ -846,13 +858,18 @@ bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aComm
         {
             wxBell();
         }
-        else if( evt->IsAction( &EE_ACTIONS::rotateCW ) )
+        else if( evt->IsAction( &SCH_ACTIONS::rotateCW ) )
         {
-            m_toolMgr->RunSynchronousAction( EE_ACTIONS::rotateCW, aCommit );
+            m_toolMgr->RunSynchronousAction( SCH_ACTIONS::rotateCW, aCommit );
         }
-        else if( evt->IsAction( &EE_ACTIONS::rotateCCW ) )
+        else if( evt->IsAction( &SCH_ACTIONS::rotateCCW ) )
         {
-            m_toolMgr->RunSynchronousAction( EE_ACTIONS::rotateCCW, aCommit );
+            m_toolMgr->RunSynchronousAction( SCH_ACTIONS::rotateCCW, aCommit );
+        }
+        else if( evt->IsAction( &ACTIONS::increment ) )
+        {
+            m_toolMgr->RunSynchronousAction( ACTIONS::increment, aCommit,
+                                             evt->Parameter<ACTIONS::INCREMENT>() );
         }
         else if( evt->Action() == TA_CHOICE_MENU_CHOICE )
         {
@@ -868,9 +885,21 @@ bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aComm
                     m_toolMgr->PostAction( ACTIONS::refreshPreview );
                 }
             }
+            else if( *evt->GetCommandId() >= ID_POPUP_SCH_SELECT_BASE
+                     && *evt->GetCommandId() <= ID_POPUP_SCH_SELECT_ALT )
+            {
+                SCH_SYMBOL* symbol = dynamic_cast<SCH_SYMBOL*>( selection.Front() );
+                int bodyStyle = ( *evt->GetCommandId() - ID_POPUP_SCH_SELECT_BASE ) + 1;
+
+                if( symbol && symbol->GetBodyStyle() != bodyStyle )
+                {
+                    m_frame->FlipBodyStyle( symbol );
+                    m_toolMgr->PostAction( ACTIONS::refreshPreview );
+                }
+            }
         }
-        else if( evt->IsAction( &EE_ACTIONS::highlightNet )
-                    || evt->IsAction( &EE_ACTIONS::selectOnPCB ) )
+        else if( evt->IsAction( &SCH_ACTIONS::highlightNet )
+                    || evt->IsAction( &SCH_ACTIONS::selectOnPCB ) )
         {
             // These don't make any sense during a move.  Eat them.
         }
@@ -879,7 +908,7 @@ bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aComm
         //
         else if( evt->IsClick( BUT_RIGHT ) )
         {
-            m_menu.ShowContextMenu( m_selectionTool->GetSelection() );
+            m_menu->ShowContextMenu( m_selectionTool->GetSelection() );
         }
         //------------------------------------------------------------------------
         // Handle drop
@@ -902,7 +931,7 @@ bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aComm
     // Create a selection of original selection, drag selected/changed items, and new
     // bend lines for later before we clear them in the aCommit. We'll need these
     // to check for new junctions needed, etc.
-    EE_SELECTION selectionCopy( selection );
+    SCH_SELECTION selectionCopy( selection );
 
     for( SCH_LINE* line : m_newDragLines )
         selectionCopy.Add( line );
@@ -985,7 +1014,7 @@ bool SCH_MOVE_TOOL::doMoveSelection( const TOOL_EVENT& aEvent, SCH_COMMIT* aComm
         item->ClearEditFlags();
 
     if( unselect )
-        m_toolMgr->RunAction( EE_ACTIONS::clearSelection );
+        m_toolMgr->RunAction( SCH_ACTIONS::clearSelection );
     else
         m_selectionTool->RebuildSelection();  // Schematic cleanup might have merged lines, etc.
 
@@ -1575,7 +1604,7 @@ void SCH_MOVE_TOOL::moveItem( EDA_ITEM* aItem, const VECTOR2I& aDelta )
 
         // If we're moving a field with respect to its parent then it's no longer auto-placed
         if( aItem->Type() == SCH_FIELD_T && parent && !parent->IsSelected() )
-            parent->ClearFieldsAutoplaced();
+            parent->SetFieldsAutoplaced( AUTOPLACE_NONE );
 
         break;
     }
@@ -1622,7 +1651,7 @@ void SCH_MOVE_TOOL::moveItem( EDA_ITEM* aItem, const VECTOR2I& aDelta )
 int SCH_MOVE_TOOL::AlignToGrid( const TOOL_EVENT& aEvent )
 {
     EE_GRID_HELPER    grid( m_toolMgr);
-    EE_SELECTION&     selection = m_selectionTool->RequestSelection( EE_COLLECTOR::MovableItems );
+    SCH_SELECTION&    selection = m_selectionTool->RequestSelection( SCH_COLLECTOR::MovableItems );
     GRID_HELPER_GRIDS selectionGrid = grid.GetSelectionGrid( selection );
     SCH_COMMIT        commit( m_toolMgr );
 
@@ -1811,9 +1840,9 @@ void SCH_MOVE_TOOL::clearNewDragLines()
 
 void SCH_MOVE_TOOL::setTransitions()
 {
-    Go( &SCH_MOVE_TOOL::Main,               EE_ACTIONS::move.MakeEvent() );
-    Go( &SCH_MOVE_TOOL::Main,               EE_ACTIONS::drag.MakeEvent() );
-    Go( &SCH_MOVE_TOOL::AlignToGrid,        EE_ACTIONS::alignToGrid.MakeEvent() );
+    Go( &SCH_MOVE_TOOL::Main,               SCH_ACTIONS::move.MakeEvent() );
+    Go( &SCH_MOVE_TOOL::Main,               SCH_ACTIONS::drag.MakeEvent() );
+    Go( &SCH_MOVE_TOOL::AlignToGrid,        SCH_ACTIONS::alignToGrid.MakeEvent() );
 }
 
 
