@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2020 Ian McInerney <Ian.S.McInerney at ieee.org>
- * Copyright (C) 2020 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -163,6 +163,11 @@ namespace KIPLATFORM
          */
         void ImmControl( wxWindow* aWindow, bool aEnable );
 
+        /**
+         * Asks the IME to cancel
+         */
+        void ImeNotifyCancelComposition( wxWindow* aWindow );
+
         /**
          * On Wayland, restricts the pointer movement to a rectangle slightly bigger than the given `wxWindow`.
          * This way, the cursor doesn't exit the (bigger) application window and we retain control on it.
