@@ -424,6 +424,12 @@ public:
                                                               PCB_GENERATOR_T, PCB_FOOTPRINT_T,
                                                               PCB_TRACE_T, PCB_SHAPE_T } );
 
+    /**
+     * Remove all teardrop zones with the STRUCT_DELETED flag set.  This avoids O(n^2) traversal
+     * over the zone list.
+     */
+    void BulkRemoveStaleTeardrops( BOARD_COMMIT& aCommit );
+
     /**
      * Must be used if Add() is used using a BULK_x ADD_MODE to generate a change event for
      * listeners.
@@ -597,6 +603,7 @@ public:
      * @return the enabled layers in bit-mapped form.
      */
     LSET GetEnabledLayers() const;
+    LSET GetLayerSet() const override { return GetEnabledLayers(); }
 
     /**
      * A proxy function that calls the correspondent function in m_BoardSettings.
@@ -604,6 +611,7 @@ public:
      * @param aLayerMask the new bit-mask of enabled layers.
      */
     void SetEnabledLayers( LSET aLayerMask );
+    void SetLayerSet( const LSET& aLayerMask ) override { SetEnabledLayers( aLayerMask ); }
 
     /**
      * A proxy function that calls the correspondent function in m_BoardSettings
