@@ -358,26 +358,28 @@ void BRDITEMS_PLOTTER::PlotFootprintTextItems( const FOOTPRINT* aFootprint )
     if( !GetPlotFPText() )
         return;
 
-    const PCB_TEXT* textItem = &aFootprint->Reference();
-    PCB_LAYER_ID    textLayer = textItem->GetLayer();
+    const PCB_TEXT* reference = &aFootprint->Reference();
+    PCB_LAYER_ID    refLayer = reference->GetLayer();
 
     // Reference and value have special controls for forcing their plotting
-    if( GetPlotReference() && m_layerMask[textLayer]
-            && ( textItem->IsVisible() || GetPlotInvisibleText() )
-            && !( aFootprint->IsDNP() && hideDNPItems( textLayer ) ) )
+    if( GetPlotReference()
+            && m_layerMask[refLayer]
+            && reference->IsVisible()
+            && !( aFootprint->IsDNP() && hideDNPItems( refLayer ) ) )
     {
-        PlotText( textItem, textLayer, textItem->IsKnockout(), textItem->GetFontMetrics(),
-                  aFootprint->IsDNP() && crossoutDNPItems( textLayer ) );
+        PlotText( reference, refLayer, reference->IsKnockout(), reference->GetFontMetrics(),
+                  aFootprint->IsDNP() && crossoutDNPItems( refLayer ) );
     }
 
-    textItem  = &aFootprint->Value();
-    textLayer = textItem->GetLayer();
+    const PCB_TEXT* value  = &aFootprint->Value();
+    PCB_LAYER_ID    valueLayer = value->GetLayer();
 
-    if( GetPlotValue() && m_layerMask[textLayer]
-            && ( textItem->IsVisible() || GetPlotInvisibleText() )
-            && !( aFootprint->IsDNP() && hideDNPItems( textLayer ) ) )
+    if( GetPlotValue()
+            && m_layerMask[valueLayer]
+            && value->IsVisible()
+            && !( aFootprint->IsDNP() && hideDNPItems( valueLayer ) ) )
     {
-        PlotText( textItem, textLayer, textItem->IsKnockout(), textItem->GetFontMetrics(),
+        PlotText( value, valueLayer, value->IsKnockout(), value->GetFontMetrics(),
                   false );
     }
 
@@ -394,10 +396,8 @@ void BRDITEMS_PLOTTER::PlotFootprintTextItems( const FOOTPRINT* aFootprint )
 
     for( BOARD_ITEM* item : aFootprint->GraphicalItems() )
     {
-        textItem = dynamic_cast<const PCB_TEXT*>( item );
-
-        if( textItem )
-            texts.push_back( static_cast<PCB_TEXT*>( item ) );
+        if( PCB_TEXT* textItem = dynamic_cast<PCB_TEXT*>( item ) )
+            texts.push_back( textItem );
     }
 
     for( const PCB_TEXT* text : texts )
@@ -405,8 +405,8 @@ void BRDITEMS_PLOTTER::PlotFootprintTextItems( const FOOTPRINT* aFootprint )
         if( !text->IsVisible() )
             continue;
 
-        textLayer = text->GetLayer();
-        bool strikeout = false;
+        PCB_LAYER_ID textLayer = text->GetLayer();
+        bool         strikeout = false;
 
         if( textLayer == Edge_Cuts || textLayer >= PCB_LAYER_ID_COUNT )
             continue;
@@ -1098,10 +1098,6 @@ void BRDITEMS_PLOTTER::PlotTableBorders( const PCB_TABLE* aTable )
     if( !m_layerMask[aTable->GetLayer()] )
         return;
 
-    VECTOR2I     pos = aTable->GetPosition();
-    VECTOR2I     end = aTable->GetEnd();
-    int          lineWidth;
-    LINE_STYLE   lineStyle;
     GBR_METADATA gbr_metadata;
 
     if( const FOOTPRINT* parentFP = aTable->GetParentFootprint() )
@@ -1110,27 +1106,12 @@ void BRDITEMS_PLOTTER::PlotTableBorders( const PCB_TABLE* aTable )
         gbr_metadata.SetNetAttribType( GBR_NETLIST_METADATA::GBR_NETINFO_CMP );
     }
 
-    auto setupStroke =
-            [&]( const STROKE_PARAMS& stroke )
+    aTable->DrawBorders(
+            [&]( const VECTOR2I& ptA, const VECTOR2I& ptB, const STROKE_PARAMS& stroke )
             {
-                lineWidth = stroke.GetWidth();
-                lineStyle = stroke.GetLineStyle();
-            };
+                int        lineWidth = stroke.GetWidth();
+                LINE_STYLE lineStyle = stroke.GetLineStyle();
 
-    auto strokeShape =
-            [&]( const SHAPE& shape )
-            {
-                STROKE_PARAMS::Stroke( &shape, lineStyle, lineWidth, m_plotter->RenderSettings(),
-                        [&]( const VECTOR2I& a, const VECTOR2I& b )
-                        {
-                            m_plotter->ThickSegment( a, b, lineWidth, GetPlotMode(),
-                                                     &gbr_metadata );
-                        } );
-            };
-
-    auto strokeLine =
-            [&]( const VECTOR2I& ptA, const VECTOR2I& ptB )
-            {
                 if( lineStyle <= LINE_STYLE::FIRST_TYPE )
                 {
                     m_plotter->ThickSegment( ptA, ptB, lineWidth, GetPlotMode(), &gbr_metadata );
@@ -1138,75 +1119,15 @@ void BRDITEMS_PLOTTER::PlotTableBorders( const PCB_TABLE* aTable )
                 else
                 {
                     SHAPE_SEGMENT seg( ptA, ptB );
-                    strokeShape( seg );
+
+                    STROKE_PARAMS::Stroke( &seg, lineStyle, lineWidth, m_plotter->RenderSettings(),
+                            [&]( const VECTOR2I& a, const VECTOR2I& b )
+                            {
+                                m_plotter->ThickSegment( a, b, lineWidth, GetPlotMode(),
+                                                         &gbr_metadata );
+                            } );
                 }
-            };
-
-    auto strokeRect =
-            [&]( const VECTOR2I& ptA, const VECTOR2I& ptB )
-            {
-                strokeLine( VECTOR2I( ptA.x, ptA.y ), VECTOR2I( ptB.x, ptA.y ) );
-                strokeLine( VECTOR2I( ptB.x, ptA.y ), VECTOR2I( ptB.x, ptB.y ) );
-                strokeLine( VECTOR2I( ptB.x, ptB.y ), VECTOR2I( ptA.x, ptB.y ) );
-                strokeLine( VECTOR2I( ptA.x, ptB.y ), VECTOR2I( ptA.x, ptA.y ) );
-            };
-
-    if( aTable->GetSeparatorsStroke().GetWidth() >= 0 )
-    {
-        setupStroke( aTable->GetSeparatorsStroke() );
-
-        if( aTable->StrokeColumns() )
-        {
-            for( int col = 0; col < aTable->GetColCount() - 1; ++col )
-            {
-                for( int row = 0; row < aTable->GetRowCount(); ++row )
-                {
-                    PCB_TABLECELL* cell = aTable->GetCell( row, col );
-                    VECTOR2I       topRight( cell->GetEndX(), cell->GetStartY() );
-
-                    if( cell->GetColSpan() > 0 && cell->GetRowSpan() > 0 )
-                        strokeLine( topRight, cell->GetEnd() );
-                }
-            }
-        }
-
-        if( aTable->StrokeRows() )
-        {
-            for( int row = 0; row < aTable->GetRowCount() - 1; ++row )
-            {
-                for( int col = 0; col < aTable->GetColCount(); ++col )
-                {
-                    PCB_TABLECELL* cell = aTable->GetCell( row, col );
-                    VECTOR2I       botLeft( cell->GetStartX(), cell->GetEndY() );
-
-                    if( cell->GetColSpan() > 0 && cell->GetRowSpan() > 0 )
-                        strokeLine( botLeft, cell->GetEnd() );
-                }
-            }
-        }
-    }
-
-    if( aTable->GetBorderStroke().GetWidth() >= 0 )
-    {
-        setupStroke( aTable->GetBorderStroke() );
-        PCB_TABLECELL* cell = aTable->GetCell( 0, 0 );
-
-        if( aTable->StrokeHeader() )
-        {
-            if( !cell->GetTextAngle().IsHorizontal() )
-                strokeLine( VECTOR2I( cell->GetEndX(), pos.y ), VECTOR2I( cell->GetEndX(), cell->GetEndY() ) );
-            else
-                strokeLine( VECTOR2I( pos.x, cell->GetEndY() ), VECTOR2I( end.x, cell->GetEndY() ) );
-        }
-
-        if( aTable->StrokeExternal() )
-        {
-            RotatePoint( pos, aTable->GetPosition(), cell->GetTextAngle() );
-            RotatePoint( end, aTable->GetPosition(), cell->GetTextAngle() );
-
-            strokeRect( pos, end );
-        }
-    }
+            } );
 }
 
 
